@@ -6,6 +6,7 @@ public class A3ControllerTest extends KeyAdapter {
     public A3ControllerTest() throws Exception {
         window = new A3Window(300,300);
         window.setCameraLocImmediately(0.0,0.0,5.0);
+        window.setDefaultCameraLoc(0.0,0.0,5.0);
         Action3D a3_1 = Action3D.load("x-res:///axis.a3");
         window.add(a3_1);
         Action3D a3_2 = Action3D.load("x-res:///axis.a3");
@@ -13,6 +14,7 @@ public class A3ControllerTest extends KeyAdapter {
         window.add(a3_2);
         window.addKeyListener(this);
         System.out.println("Press [n|w|e|f|d|s|c] key to change NaviMode.");
+        System.out.println("Press r key to resetCamera.");
     }
     public void keyTyped(KeyEvent e) {
         System.out.println("'"+e.getKeyChar()+"' pressed.");
@@ -30,6 +32,8 @@ public class A3ControllerTest extends KeyAdapter {
             window.setNavigationMode(A3CanvasInterface.NaviMode.SIMPLE);
         else if (e.getKeyChar()=='c')
             window.setNavigationMode(A3CanvasInterface.NaviMode.CHASE);
+        else if (e.getKeyChar()=='r')
+            window.resetCamera();
     }
     public static void main(String args[]) throws Exception {
         new A3ControllerTest();
