@@ -2,8 +2,9 @@ import jp.sourceforge.acerola3d.a3.*;
 
 public class ManySound {
     public static void main(String args[]) throws Exception {
-        System.setProperty("j3d.audiodevice","com.sun.j3d.audioengines.javasound.JavaSoundMixer");
+        //System.setProperty("j3d.audiodevice","com.sun.j3d.audioengines.javasound.JavaSoundMixer");
         //System.setProperty("j3d.audiodevice","org.jdesktop.j3d.audioengines.joal.JOALMixer");
+        //System.setProperty("acerola3d.action3d.sound","JOAL_DIRECT");
         A3Window w = new A3Window(300,300);
         Action3D a3s[][] = new Action3D[10][10];
         for (int i=0;i<10;i++) {
