@@ -53,8 +53,8 @@ public class A23 {
                 });
             } catch(Error e) {
                 //lg3dで使うような場合のためにこれを追加．
-                System.out.println("A23.initA23(). A23URLStreamHandlerFactory is disabled.");
-                e.printStackTrace();
+                System.out.println("A23.initA23(). URLStreamHandlerFactory has already set.");
+                //e.printStackTrace();
             }
             isInitialized = true;
         }
