@@ -56,6 +56,9 @@ class A3Behavior extends Behavior {
         }
         t.set(tmpQ,nowV,nowS);
         transGroup.setTransform(t);
+        if (a3 instanceof Action3D) {
+            ((Action3D)a3).setSoundLocQuat(nowV,tmpQ);
+        }
     }
     public void initialize() {
         WakeupOnElapsedTime w = new WakeupOnElapsedTime(elapsedTime);
@@ -100,6 +103,9 @@ class A3Behavior extends Behavior {
         }
         t.set(tmpQ,nowV,nowS);
         transGroup.setTransform(t);
+        if (a3 instanceof Action3D) {
+            ((Action3D)a3).setSoundLocQuat(nowV,tmpQ);
+        }
         if (!isInterpolate) {
             needRecalc = false;
         }
@@ -218,6 +224,9 @@ class A3Behavior extends Behavior {
         }
         t.set(tmpQ,nowV,nowS);
         transGroup.setTransform(t);
+        if (a3 instanceof Action3D) {
+            ((Action3D)a3).setSoundLocQuat(nowV,tmpQ);
+        }
 //        needRecalc = true;
     }
     void setLoc(Vector3d v) {
@@ -243,6 +252,9 @@ class A3Behavior extends Behavior {
         }
         t.set(tmpQ,nowV,nowS);
         transGroup.setTransform(t);
+        if (a3 instanceof Action3D) {
+            ((Action3D)a3).setSoundLocQuat(nowV,tmpQ);
+        }
 //        needRecalc = true;
     }
     Vector3d getLoc() {
@@ -271,6 +283,9 @@ class A3Behavior extends Behavior {
         }
         t.set(tmpQ,nowV,nowS);
         transGroup.setTransform(t);
+        if (a3 instanceof Action3D) {
+            ((Action3D)a3).setSoundLocQuat(nowV,tmpQ);
+        }
 //        needRecalc = true;
     }
     Quat4d getQuat() {
@@ -305,6 +320,9 @@ class A3Behavior extends Behavior {
         }
         t.set(tmpQ,nowV,nowS);
         transGroup.setTransform(t);
+        if (a3 instanceof Action3D) {
+            ((Action3D)a3).setSoundLocQuat(nowV,tmpQ);
+        }
 //        needRecalc = true;
     }
     void setScale(double s) {
@@ -330,6 +348,9 @@ class A3Behavior extends Behavior {
         }
         t.set(tmpQ,nowV,nowS);
         transGroup.setTransform(t);
+        if (a3 instanceof Action3D) {
+            ((Action3D)a3).setSoundLocQuat(nowV,tmpQ);
+        }
 //        needRecalc = true;
     }
     double getScale() {
