@@ -25,10 +25,15 @@ public interface A3CanvasInterface {
     public void del(A3Object a);
 
     /**
-     * 登録されている全てのA3Objectを削除して表示されないようにします。
-     *
+     * アクティブなシーンに登録されている全てのA3Objectを削除して表示されないようにします。
+     * ただしaddLockedA3で追加した固定オブジェクトは削除されません。
      */
     public void delAll();
+    /**
+     * シーンを指定して、登録されている全てのA3Objectを削除して表示されないようにします。
+     * ただしaddLockedA3で追加した固定オブジェクトは削除されません。
+     */
+    public void delAll(int scene);
 
     /**
      * 背景を表すA3Objectをセットします。
