@@ -15,6 +15,8 @@ import jp.sourceforge.acerola3d.a3.A3CanvasInterface.NaviMode;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 
+import jp.sourceforge.acerola3d.sound.*;
+
 /**
  * A3CanvasとJA3Canvasの両方で共通して使うことが
  * できるVirtualUniverseのAcerola3Dバージョン。
@@ -116,7 +118,7 @@ class A3VirtualUniverse extends VirtualUniverse implements A3CanvasInterface {
             }
         });
 
-        System.out.println(mixerClassName);
+        //System.out.println(mixerClassName);
         if (mixerClassName!=null) {
             try {
                 Class<?> c = Class.forName(mixerClassName);
@@ -127,6 +129,20 @@ class A3VirtualUniverse extends VirtualUniverse implements A3CanvasInterface {
                 e.printStackTrace();
             }
         }
+        if (mixer==null) {
+            try {
+                mixer = new com.sun.j3d.audioengines.javasound.JavaSoundMixer(environment);
+            } catch(Exception e) {
+                ;
+            }
+        }
+        if (mixer==null) {
+            try {
+                mixer = new jp.sourceforge.acerola3d.sound.JOALMixer2(environment);
+            } catch(Exception e) {
+                ;
+            }
+        }
         if (mixer==null) {
             try {
                 mixer = new org.jdesktop.j3d.audioengines.joal.JOALMixer(environment);
@@ -143,17 +159,10 @@ class A3VirtualUniverse extends VirtualUniverse implements A3CanvasInterface {
             }
         }
         */
-        if (mixer==null) {
-            try {
-                mixer = new com.sun.j3d.audioengines.javasound.JavaSoundMixer(environment);
-            } catch(Exception e) {
-                ;
-            }
-        }
         if (mixer == null) {
             System.out.println("null AudioDevice!");
         } else {
-            System.out.println(mixer.getClass().getName());
+            System.out.println("j3d.audiodevice="+mixer.getClass().getName());
             environment.setAudioDevice(mixer);
             try {
                 mixer.initialize();
@@ -162,6 +171,19 @@ class A3VirtualUniverse extends VirtualUniverse implements A3CanvasInterface {
             }
         }
 
+        String action3dSoundType = 
+        AccessController.doPrivilegedWithCombiner(new PrivilegedAction<String>() {
+            public String run() {
+                return System.getProperty("acerola3d.action3d.sound");
+            }
+        });
+        if ((action3dSoundType==null)||(!action3dSoundType.equals("J3D_AUDIODEVICE"))) {
+            Action3DData.soundSystem = A3SoundSystem.getDefaultSoundSystem();
+            System.out.println("acerola3d.action3d.sound=JOAL_DIRECT");
+        } else {
+            System.out.println("acerola3d.action3d.sound=J3D_AUDIODEVICE");
+        }
+
         scene = new Scene(this,0);
         scenes.put(0,scene);
         //rootGroup.addChild(scene.mainGroup);
@@ -192,13 +214,25 @@ class A3VirtualUniverse extends VirtualUniverse implements A3CanvasInterface {
     }
 
     /**
-     * 登録されている全てのA3Objectを削除して表示されないようにします。
-     *
+     * アクティブなシーンに登録されている全てのA3Objectを削除して表示されないようにします。
+     * ただしaddLockedA3で追加した固定オブジェクトには削除されません。
      */
     public void delAll() {
         scene.delAll();
     }
 
+    /**
+     * シーンを指定して、登録されている全てのA3Objectを削除して表示されないようにします。
+     * ただしaddLockedA3で追加した固定オブジェクトは削除されません。
+     */
+    public void delAll(int scene) {
+        Scene s = scenes.get(scene);
+        if (s!=null)
+            s.delAll();
+        else
+            throw new IllegalArgumentException();
+    }
+
     /**
      * 背景を表すA3Objectをセットします。
      */
