@@ -94,13 +94,21 @@ public class A3Widget extends JFrame implements A3CanvasInterface {
     }
 
     /**
-     * 登録されている全てのA3Objectを削除して表示されないようにします。
-     *
+     * アクティブなシーンに登録されている全てのA3Objectを削除して表示されないようにします。
+     * ただしaddLockedA3で追加した固定オブジェクトには削除されません。
      */
     public void delAll() {
         universe.delAll();
     }
 
+    /**
+     * シーンを指定して、登録されている全てのA3Objectを削除して表示されないようにします。
+     * ただしaddLockedA3で追加した固定オブジェクトは削除されません。
+     */
+    public void delAll(int scene) {
+        universe.delAll(scene);
+    }
+
     /**
      * 背景を表すA3Objectをセットします。
      */
