@@ -3,6 +3,7 @@ package jp.sourceforge.acerola3d.a3;
 import java.io.*;
 import java.net.*;
 import javax.media.j3d.*;
+import javax.vecmath.*;
 
 import jp.sourceforge.acerola3d.A23;
 import jp.sourceforge.acerola3d.SoftRefMap;
@@ -544,6 +545,10 @@ public class Action3D extends A3Object implements ActionObject {
             e.printStackTrace();
         }
     }
+    public void dispose() {
+        super.dispose();
+        actionBehavior.dispose();
+    }
 //----------------------------------------
     /**
      * ラベルをセットします。(デスクトップのアイコンの
@@ -694,4 +699,9 @@ public class Action3D extends A3Object implements ActionObject {
         }
         setBalloonDir(actionNo,d);
     }
+    //----------------------------------------
+    void setSoundLocQuat(Vector3d v,Quat4d q) {
+        if (actionBehavior!=null)
+            actionBehavior.setSoundLocQuat(v,q);
+    }
 }
