@@ -34,6 +34,8 @@ import javax.xml.bind.*;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 
+import jp.sourceforge.acerola3d.sound.*;
+
 /**
  * ファイルから読み込んだデータを再利用可能な形で保存しておく
  * ためのデータ構造。さらに、このデータはコピー可能であり、
@@ -42,6 +44,7 @@ import java.security.PrivilegedAction;
 class Action3DData {
     static VrmlLoader loader = null;
     static boolean isInitialized = false;
+    static A3SoundSystem soundSystem = null;
 
     static void initAction3DData() {
         if (isInitialized==true)
@@ -80,7 +83,7 @@ class Action3DData {
     String actionNames[];
     HashMap<String,ShapeAndBF> shapes;
     HashMap<String,Motion> motions;
-    HashMap<String,MediaContainer> sounds;
+    HashMap<String,Object> sounds;
     BranchGroup allActionBranchGroup;
     //Switch allActionSwitch;
 
@@ -101,12 +104,13 @@ class Action3DData {
         boolean loopFlag;
         String bvhFile;
         String soundFile;
+        boolean soundLoop;
         String soundType = "PointSound";
         float soundGain = 1.0f;
         float soundOffsetXYZ[] = {0.0f,0.0f,0.0f};
         float soundDirectionXYZ[] = {0.0f,0.0f,1.0f};
         boolean soundContinue = true;
-        Sound sound;
+        Object sound;
         double scale;
         double[] offsetXYZ;
         double[] rotationXYZ;
@@ -132,6 +136,7 @@ class Action3DData {
             ret.loopFlag = loopFlag;
             ret.bvhFile = bvhFile;
             ret.soundFile = soundFile;
+            ret.soundLoop = soundLoop;
             ret.soundType = soundType;
             ret.soundGain = soundGain;
             ret.soundOffsetXYZ = soundOffsetXYZ.clone();
@@ -167,17 +172,21 @@ class Action3DData {
             } else {
                 Transform3D t0a = new Transform3D();
                 Transform3D t0b = new Transform3D();
-                if (rotationXYZ!=null) {
-                    t0a.rotX(rotationXYZ[0]/180.0*2.0*Math.PI);
-                    t0b.rotY(rotationXYZ[1]/180.0*2.0*Math.PI);
-                    t0a.mul(t0b);
-                    t0b.rotZ(rotationXYZ[2]/180.0*2.0*Math.PI);
+                if (offsetXYZ!=null) {
+                    t0b.setTranslation(new Vector3d(offsetXYZ[0],offsetXYZ[1],offsetXYZ[2]));
                     t0a.mul(t0b);
                 }
-                t0b.set(Transform3D.IDENTITY);
-                t0b.setScale(scale);//gaha
-                if (offsetXYZ!=null)
-                    t0b.setTranslation(new Vector3d(offsetXYZ[0],offsetXYZ[1],offsetXYZ[2]));
+                if (rotationXYZ!=null) {
+                    t0b.setIdentity();
+                    t0b.rotX(rotationXYZ[0]/180.0*Math.PI);
+                    t0a.mul(t0b);
+                    t0b.rotY(rotationXYZ[1]/180.0*Math.PI);
+                    t0a.mul(t0b);
+                    t0b.rotZ(rotationXYZ[2]/180.0*Math.PI);
+                    t0a.mul(t0b);
+                }
+                t0b.setIdentity();
+                t0b.setScale(scale);
                 t0a.mul(t0b);
                 tg0 = new TransformGroup(t0a);
                 tg0.addChild(tg);
@@ -200,23 +209,22 @@ class Action3DData {
                 } else {
                     Transform3D t9a = new Transform3D();
                     Transform3D t9b = new Transform3D();
-                    t9a.set(Transform3D.IDENTITY);
-                    t9b.set(Transform3D.IDENTITY);
+                    if (p.offsetXYZ!=null) {
+                        t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
+                        t9a.mul(t9b);
+                    }
                     if (p.rotationXYZ!=null) {
-                        t9a.rotX(p.rotationXYZ[0]/180.0*Math.PI);
+                        t9b.setIdentity();
+                        t9b.rotX(p.rotationXYZ[0]/180.0*Math.PI);
+                        t9a.mul(t9b);
                         t9b.rotY(p.rotationXYZ[1]/180.0*Math.PI);
                         t9a.mul(t9b);
                         t9b.rotZ(p.rotationXYZ[2]/180.0*Math.PI);
                         t9a.mul(t9b);
                     }
-                    t9b.set(Transform3D.IDENTITY);
+                    t9b.setIdentity();
                     t9b.setScale(p.scale);
                     t9a.mul(t9b);
-                    if (p.offsetXYZ!=null) {
-                        t9b.set(Transform3D.IDENTITY);
-                        t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
-                        t9a.mul(t9b);
-                    }
                     TransformGroup tg9 = new TransformGroup(t9a);
                     tg9.addChild(new Link(sabf.sg));
                     n = tg9;
@@ -236,78 +244,103 @@ class Action3DData {
             construct(tg,rootBoneName);
 
             if (soundFile!=null) {
-                MediaContainer mc = sounds.get(soundFile);
-                if (soundType.equals("BackgroundSound")) {
-                    sound = new BackgroundSound(mc,soundGain);
-                } else if (soundType.equals("PointSound")){
-                    sound = new PointSound(mc,soundGain,
-                                           soundOffsetXYZ[0],
-                                           soundOffsetXYZ[1],
-                                           soundOffsetXYZ[2]);
-                } else if (soundType.equals("ConeSound")) {
-                    /*
-                    sound = new ConeSound(mc,soundGain,
-                            soundOffsetXYZ[0],
-                            soundOffsetXYZ[1],
-                            soundOffsetXYZ[2],
-                            soundDirectionXYZ[0],
-                            soundDirectionXYZ[1],
-                            soundDirectionXYZ[2]);
-                    */
+                Object s = sounds.get(soundFile);
+                if (s instanceof MediaContainer) {
+                    MediaContainer mc = (MediaContainer)s;
+                    if (soundType.equals("BackgroundSound")) {
+                        sound = new BackgroundSound(mc,soundGain);
+                    } else if (soundType.equals("PointSound")){
+                        sound = new PointSound(mc,soundGain,
+                                               soundOffsetXYZ[0],
+                                               soundOffsetXYZ[1],
+                                               soundOffsetXYZ[2]);
+                    } else if (soundType.equals("ConeSound")) {
+                        /*
+                        sound = new ConeSound(mc,soundGain,
+                                soundOffsetXYZ[0],
+                                soundOffsetXYZ[1],
+                                soundOffsetXYZ[2],
+                                soundDirectionXYZ[0],
+                                soundDirectionXYZ[1],
+                                soundDirectionXYZ[2]);
+                        */
 
-                    //http://old.siggraph.org/education/materials/siggraph_courses/S99/40/java3d4p.pdf
-                    //さっぱり意味がわかりません。
-                    Point2f[] myFrontAtten = {
-                            new Point2f( 100.0f, 1.0f ),
-                            new Point2f( 350.0f, 0.5f ),
-                            new Point2f( 600.0f, 0.0f )
-                            };
-                            Point2f[] myBackAtten = {
-                            new Point2f( 50.0f, 1.0f ),
-                            new Point2f( 100.0f, 0.5f ),
-                            new Point2f( 200.0f, 0.0f )
-                            };
-                            Point3f[] myAngular = {
-                            new Point3f( 0.000f, 1.0f, 20000.0f ),
-                            new Point3f( 0.785f, 0.5f, 5000.0f ),
-                            new Point3f( 1.571f, 0.0f, 2000.0f ),
-                            };
-                    sound = new ConeSound();
-                    ((ConeSound)sound).setSoundData(mc);
-                    ((ConeSound)sound).setInitialGain(soundGain);
-                    ((ConeSound)sound).setPosition(soundOffsetXYZ[0],soundOffsetXYZ[1],soundOffsetXYZ[2]);
-                    ((ConeSound)sound).setDirection(soundDirectionXYZ[0],soundDirectionXYZ[1],soundDirectionXYZ[2]);
-                    ((ConeSound)sound).setDistanceGain(myFrontAtten,myBackAtten);
-                    ((ConeSound)sound).setAngularAttenuation(myAngular);
+                        //http://old.siggraph.org/education/materials/siggraph_courses/S99/40/java3d4p.pdf
+                        //さっぱり意味がわかりません。
+                        Point2f[] myFrontAtten = {
+                                new Point2f( 100.0f, 1.0f ),
+                                new Point2f( 350.0f, 0.5f ),
+                                new Point2f( 600.0f, 0.0f )
+                                };
+                                Point2f[] myBackAtten = {
+                                new Point2f( 50.0f, 1.0f ),
+                                new Point2f( 100.0f, 0.5f ),
+                                new Point2f( 200.0f, 0.0f )
+                                };
+                                Point3f[] myAngular = {
+                                new Point3f( 0.000f, 1.0f, 20000.0f ),
+                                new Point3f( 0.785f, 0.5f, 5000.0f ),
+                                new Point3f( 1.571f, 0.0f, 2000.0f ),
+                                };
+                        sound = new ConeSound();
+                        ((ConeSound)sound).setSoundData(mc);
+                        ((ConeSound)sound).setInitialGain(soundGain);
+                        ((ConeSound)sound).setPosition(soundOffsetXYZ[0],soundOffsetXYZ[1],soundOffsetXYZ[2]);
+                        ((ConeSound)sound).setDirection(soundDirectionXYZ[0],soundDirectionXYZ[1],soundDirectionXYZ[2]);
+                        ((ConeSound)sound).setDistanceGain(myFrontAtten,myBackAtten);
+                        ((ConeSound)sound).setAngularAttenuation(myAngular);
 
-                    /*
-                    //http://www.koders.com/java/fid9234663F18A13A2F1B99E463FCFE7B1D75BCB621.aspx?s=DistanceAttenuation#L40
-                    //さっぱり意味がわかりません。
-                    sound = new ConeSound(mc,soundGain,
-                            soundOffsetXYZ[0],
-                            soundOffsetXYZ[1],
-                            soundOffsetXYZ[2],
-                            soundDirectionXYZ[0],
-                            soundDirectionXYZ[1],
-                            soundDirectionXYZ[2]);
-                    float angle = (float) (Math.PI / 4);
-                    ((ConeSound)sound).setAngularAttenuation(new Point2f[] {
-                        new Point2f(angle, 0)});
-                    float distanceAtZero = 40;
-                    ((ConeSound)sound).setDistanceGain(new float []{0, distanceAtZero}, new float []{1, 0});
-                    */
+                        /*
+                        //http://www.koders.com/java/fid9234663F18A13A2F1B99E463FCFE7B1D75BCB621.aspx?s=DistanceAttenuation#L40
+                        //さっぱり意味がわかりません。
+                        sound = new ConeSound(mc,soundGain,
+                                soundOffsetXYZ[0],
+                                soundOffsetXYZ[1],
+                                soundOffsetXYZ[2],
+                                soundDirectionXYZ[0],
+                                soundDirectionXYZ[1],
+                                soundDirectionXYZ[2]);
+                        float angle = (float) (Math.PI / 4);
+                        ((ConeSound)sound).setAngularAttenuation(new Point2f[] {
+                            new Point2f(angle, 0)});
+                        float distanceAtZero = 40;
+                        ((ConeSound)sound).setDistanceGain(new float []{0, distanceAtZero}, new float []{1, 0});
+                        */
+                    }
+                    Sound j3dSound = (Sound)sound;
+                    j3dSound.setEnable(false);
+                    j3dSound.setPause(false);
+                    if (soundLoop)
+                        j3dSound.setLoop(-1);
+                    else
+                        j3dSound.setLoop(0);
+                    //j3dSound.setContinuousEnable(true);
+                    j3dSound.setSchedulingBounds(bounding);
+                    j3dSound.setCapability(Sound.ALLOW_ENABLE_WRITE);
+                    //j3dSound.setCapability(Sound.ALLOW_PAUSE_WRITE);
+                    SharedGroup sg = new SharedGroup();
+                    sg.addChild(j3dSound);
+                    Link ll = new Link(sg);
+                    rootGroup.addChild(ll);
+                } else if (s instanceof A3Sound) {
+                    A3Sound a3Sound = ((A3Sound)s).copy();
+                    a3Sound.setGain(soundGain);
+                    a3Sound.setSegno((float)segno);
+                    a3Sound.setLoop(soundLoop);
+                    A3SoundType type = null;
+                    if (soundType.equals("BackgroundSound")) {
+                        type = A3SoundType.BackgroundSound;
+                        a3Sound.setType(type);
+                    } else if (soundType.equals("ConeSound")) {
+                        type = A3SoundType.ConeSound;
+                        a3Sound.setType(type);
+                        a3Sound.setDirection(soundDirectionXYZ[0],soundDirectionXYZ[1],soundDirectionXYZ[2]);
+                    } else {
+                        type = A3SoundType.PointSound;
+                        a3Sound.setType(type);
+                    }
+                    sound = a3Sound;
                 }
-                sound.setEnable(false);
-                sound.setPause(false);
-                sound.setLoop(0);
-                //sound.setContinuousEnable(true);
-                sound.setSchedulingBounds(bounding);
-                sound.setCapability(Sound.ALLOW_ENABLE_WRITE);
-                //sound.setCapability(Sound.ALLOW_PAUSE_WRITE);
-                SharedGroup sg = new SharedGroup();
-                sg.addChild(sound);
-                Link ll = new Link(sg);
-                rootGroup.addChild(ll);
             }
             //boneに関連付けされていないPartの処理
             String bones[] = motion.getAllBones();
@@ -330,23 +363,22 @@ class Action3DData {
                     } else {
                         Transform3D t9a = new Transform3D();
                         Transform3D t9b = new Transform3D();
-                        t9a.set(Transform3D.IDENTITY);
-                        t9b.set(Transform3D.IDENTITY);
+                        if (p.offsetXYZ!=null) {
+                            t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
+                            t9a.mul(t9b);
+                        }
                         if (p.rotationXYZ!=null) {
-                            t9a.rotX(p.rotationXYZ[0]/180.0*Math.PI);
+                            t9b.setIdentity();
+                            t9b.rotX(p.rotationXYZ[0]/180.0*Math.PI);
+                            t9a.mul(t9b);
                             t9b.rotY(p.rotationXYZ[1]/180.0*Math.PI);
                             t9a.mul(t9b);
                             t9b.rotZ(p.rotationXYZ[2]/180.0*Math.PI);
                             t9a.mul(t9b);
                         }
-                        t9b.set(Transform3D.IDENTITY);
+                        t9b.setIdentity();
                         t9b.setScale(p.scale);
                         t9a.mul(t9b);
-                        if (p.offsetXYZ!=null) {
-                            t9b.set(Transform3D.IDENTITY);
-                            t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
-                            t9a.mul(t9b);
-                        }
                         TransformGroup tg9 = new TransformGroup(t9a);
                         tg9.addChild(new Link(sabf.sg));
                         n = tg9;
@@ -380,23 +412,22 @@ class Action3DData {
                     } else {
                         Transform3D t9a = new Transform3D();
                         Transform3D t9b = new Transform3D();
-                        t9a.set(Transform3D.IDENTITY);
-                        t9b.set(Transform3D.IDENTITY);
+                        if (p.offsetXYZ!=null) {
+                            t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
+                            t9a.mul(t9b);
+                        }
                         if (p.rotationXYZ!=null) {
-                            t9a.rotX(p.rotationXYZ[0]/180.0*Math.PI);
+                            t9b.setIdentity();
+                            t9b.rotX(p.rotationXYZ[0]/180.0*Math.PI);
+                            t9a.mul(t9b);
                             t9b.rotY(p.rotationXYZ[1]/180.0*Math.PI);
                             t9a.mul(t9b);
                             t9b.rotZ(p.rotationXYZ[2]/180.0*Math.PI);
                             t9a.mul(t9b);
                         }
-                        t9b.set(Transform3D.IDENTITY);
+                        t9b.setIdentity();
                         t9b.setScale(p.scale);
                         t9a.mul(t9b);
-                        if (p.offsetXYZ!=null) {
-                            t9b.set(Transform3D.IDENTITY);
-                            t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
-                            t9a.mul(t9b);
-                        }
                         TransformGroup tg9 = new TransformGroup(t9a);
                         tg9.addChild(new Link(sabf.sg));
                         n = tg9;
@@ -429,12 +460,38 @@ class Action3DData {
         }
 
         void stop() {
-            //if (sound!=null) {
-            //    sound.setEnable(false);
-            //    //sound.setPause(true);
-            //}
+            if (sound!=null) {
+                if (sound instanceof Sound) {
+                    Sound s = (Sound)sound;
+                    s.setEnable(false);
+                    //mc.setPause(true);
+                } else if (sound instanceof A3Sound) {
+                    A3Sound s = (A3Sound)sound;
+                    s.stop();
+                }
+            }
         }
 
+        void soundStart() {
+            if (sound instanceof Sound) {
+                ((Sound)sound).setEnable(true);
+            } else if (sound instanceof A3Sound) {
+                ((A3Sound)sound).start();
+            }
+        }
+        void soundStop() {
+            if (sound instanceof Sound) {
+                ((Sound)sound).setEnable(false);
+            } else if (sound instanceof A3Sound) {
+                ((A3Sound)sound).stop();
+            }
+        }
+        void setSoundLocQuat(Vector3d v,Quat4d q) {
+            if (sound instanceof A3Sound) {
+                ((A3Sound)sound).setLoc(v);
+                ((A3Sound)sound).setQuat(q);
+            }
+        }
         boolean isStoped() {
             return behavior.isStoped();
         }
@@ -509,7 +566,7 @@ class Action3DData {
         }
         ret.shapes = new HashMap<String,ShapeAndBF>(shapes);
         ret.motions = new HashMap<String,Motion>(motions);
-        ret.sounds = new HashMap<String,MediaContainer>(sounds);
+        ret.sounds = new HashMap<String,Object>(sounds);
         return ret;
     }
 
@@ -631,6 +688,7 @@ class Action3DData {
             S s = aNode.getS();
             if (s!=null) {
                 a.soundFile = s.getFile();
+                a.soundLoop = s.isLoop();
                 if (s.getType()==SoundType.POINT_SOUND)
                     a.soundType = "PointSound";
                 else if (s.getType()==SoundType.BACKGROUND_SOUND)
@@ -854,6 +912,9 @@ class Action3DData {
                 nnm = s_nodes[0].getAttributes();
                 attr = (Attr)nnm.getNamedItem("file");
                 a.soundFile = attr.getValue().trim();
+                attr = (Attr)nnm.getNamedItem("loop");
+                if (attr!=null)
+                    a.soundLoop = Boolean.parseBoolean(attr.getValue().trim());
                 attr = (Attr)nnm.getNamedItem("type");
                 if (attr!=null)
                     a.soundType = attr.getValue().trim();
@@ -968,7 +1029,7 @@ class Action3DData {
         for (int i=0;i<actions.length;i++)
             if (actions[i].soundFile!=null)
                 s.add(actions[i].soundFile);
-        sounds = new HashMap<String,MediaContainer>();
+        sounds = new HashMap<String,Object>();
         for (String filename : s) {
             URL soundURL = null;
             if (url.getProtocol().equals("file")) {
@@ -981,8 +1042,17 @@ class Action3DData {
             } else {
                 soundURL = new URL("x-rzip:"+url.toString()+"!/"+filename);
             }
-            MediaContainer mc = new MediaContainer(soundURL);
-            sounds.put(filename,mc);
+            if (soundSystem==null) {
+                MediaContainer mc = new MediaContainer(soundURL);
+                sounds.put(filename,mc);
+            } else {
+                float segno = 0.0f;
+                A3SoundType type = A3SoundType.PointSound;
+                float gain = 1.0f;
+                boolean loop = false;
+                A3Sound ss = soundSystem.load(soundURL.toString(), segno, type, gain, loop);
+                sounds.put(filename,ss);
+            }
         }
     }
 
