@@ -233,20 +233,20 @@ class ActionBehavior extends Behavior implements Cloneable {
             if ((newPlayStatus>=S2)&&(playStatus<S2)) {
                 if (a3Data.actions[actionNo].sound!=null) {
                     //a3Data.actions[actionNo].sound.setEnable(false);
-                    a3Data.actions[actionNo].sound.setEnable(true);
+                    a3Data.actions[actionNo].soundStart();
                 }
             }
             //S2状態で2回目以降のサウンド再生
             if ((newPlayStatus==S2)&&(playCount!=newPlayCount)) {
                 if (a3Data.actions[actionNo].sound!=null) {
                     //a3Data.actions[actionNo].sound.setEnable(false);
-                    a3Data.actions[actionNo].sound.setEnable(true);
+                    a3Data.actions[actionNo].soundStart();
                 }
             }
             //soundContinue==falseの場合はアクション終了でサウンドも終了
             if ((newPlayStatus==S4)&&(!a3Data.actions[actionNo].soundContinue)) {
                 if (a3Data.actions[actionNo].sound!=null) {
-                    a3Data.actions[actionNo].sound.setEnable(false);
+                    a3Data.actions[actionNo].soundStop();
                 }
             }
 
@@ -320,6 +320,9 @@ class ActionBehavior extends Behavior implements Cloneable {
     public boolean isStoped() {
         return ((loopFlag==false)&&(playCount>0));
     }
+    public void dispose() {
+        a3Data.actions[actionNo].soundStop();
+    }
 
 //-------------------------------
     Node getNode() {
@@ -541,4 +544,7 @@ class ActionBehavior extends Behavior implements Cloneable {
             setBalloonDir(actionNo,d);
         }
     }
+    void setSoundLocQuat(Vector3d v,Quat4d q) {
+        a3Data.actions[actionNo].setSoundLocQuat(v,q);
+    }
 }
