@@ -25,5 +25,9 @@ class CameraBehavior extends Behavior {
         virtualUniverse.scene.cameraNowV.interpolate(virtualUniverse.scene.cameraNextV,0.2);
         virtualUniverse.transform.set(virtualUniverse.scene.cameraNowQ,virtualUniverse.scene.cameraNowV,virtualUniverse.scene.cameraNowS);
         virtualUniverse.tGroup.setTransform(virtualUniverse.transform);
+        if (Action3DData.soundSystem!=null) {
+            Action3DData.soundSystem.setListenerLoc(virtualUniverse.scene.cameraNowV);
+            Action3DData.soundSystem.setListenerQuat(virtualUniverse.scene.cameraNowQ);
+        }
     }
 }
