@@ -362,6 +362,115 @@ class Scene {
         return cameraNowS;
     }
 
+    /**
+     * ルックアットポイントを使って、カメラの回転を指定します。
+     * 上方向ベクトルはそのカメラが存在するシーンのupperDirectionの
+     * 値を用いて計算します。
+     */
+    public void setCameraLookAtPoint(Vector3d lookAt) {
+        Vector3d up = null;
+        if (upperDirection==A3Object.UpperDirection.Y)
+            up = new Vector3d(0.0,1.0,0.0);
+        else
+            up = new Vector3d(0.0,0.0,1.0);
+        setCameraLookAtPoint(lookAt,up);
+    }
+
+    /**
+     * ルックアットポイントを使って、カメラの回転を即時に指定します。
+     * 上方向ベクトルはそのカメラが存在するシーンのupperDirectionの
+     * 値を用いて計算します。
+     */
+    public void setCameraLookAtPointImmediately(Vector3d lookAt) {
+        setCameraLookAtPoint(lookAt);
+        cameraNowQ.set(cameraNextQ);
+    }
+
+    /**
+     * ルックアットポイントを使って、カメラの回転を指定します。
+     * 上方向ベクトルはそのカメラが存在するシーンのupperDirectionの
+     * 値を用いて計算します。
+     */
+    public void setCameraLookAtPoint(double x,double y,double z) {
+        Vector3d lookAt = new Vector3d(x,y,z);
+        setCameraLookAtPoint(lookAt);
+    }
+
+    /**
+     * ルックアットポイントを使って、カメラの回転を指定します。
+     * 上方向ベクトルはそのカメラが存在するシーンのupperDirectionの
+     * 値を用いて計算します。
+     */
+    public void setCameraLookAtPointImmediately(double x,double y,double z) {
+        Vector3d lookAt = new Vector3d(x,y,z);
+        setCameraLookAtPoint(lookAt);
+        cameraNowQ.set(cameraNextQ);
+    }
+
+    /**
+     * ルックアットポイントと上方向ベクトルを使って、カメラの回転を
+     * 指定します。
+     */
+    public void setCameraLookAtPoint(Vector3d lookAt,Vector3d up) {
+        Vector3d newFront = new Vector3d(cameraNowV);
+        newFront.sub(lookAt);
+        newFront.normalize();
+        //なぜかm.get(nowQ)が上手くいかないときがある。
+        //原因はよくわかってない。しょうがないので強引に
+        //以下のif文を入れる。
+        if ((Math.abs(newFront.x)<0.0001)&&(newFront.z<0.0)) {
+            newFront.x=0.0001;
+            newFront.normalize();
+        }
+        double d = newFront.dot(up);
+        Vector3d vTmp = new Vector3d(newFront);
+        vTmp.scale(d);
+        Vector3d newTop = new Vector3d(up);
+        newTop.sub(vTmp);
+        if (newTop.lengthSquared()<0.00001) {
+            //この場合とりあえず適当
+            newFront.set(0.0,0.0,1.0);
+            newTop.set(0.0,1.0,0.0);
+        } else {
+            newTop.normalize();
+        }
+        Vector3d newRight = new Vector3d();
+        newRight.cross(newFront,newTop);
+
+        Matrix4d m = new Matrix4d();
+        m.m00 = -newRight.x;m.m01 = newTop.x;m.m02 = newFront.x;
+        m.m10 = -newRight.y;m.m11 = newTop.y;m.m12 = newFront.y;
+        m.m20 = -newRight.z;m.m21 = newTop.z;m.m22 = newFront.z;
+        m.get(cameraNextQ);
+    }
+
+    /**
+     * ルックアットポイントと上方向ベクトルを使って、カメラの回転を
+     * 即時に指定します。
+     */
+    public void setCameraLookAtPointImmediately(Vector3d lookAt,Vector3d up) {
+        setCameraLookAtPoint(lookAt,up);
+        cameraNowQ.set(cameraNextQ);
+    }
+
+    /**
+     * ルックアットポイントと上方向ベクトルを使って、カメラの回転を
+     * 指定します。
+     */
+    public void setCameraLookAtPoint(double x,double y,double z,Vector3d up) {
+        Vector3d lookAt = new Vector3d(x,y,z);
+        setCameraLookAtPoint(lookAt,up);
+    }
+
+    /**
+     * ルックアットポイントと上方向ベクトルを使って、カメラの回転を
+     * 指定します。
+     */
+    public void setCameraLookAtPointImmediately(double x,double y,double z,Vector3d up) {
+        setCameraLookAtPoint(x,y,z,up);
+        cameraNowQ.set(cameraNextQ);
+    }
+
     Quat4d rot2quat(double x,double y,double z) {
         Transform3D t0 = new Transform3D();
         Transform3D t1 = new Transform3D();
@@ -423,7 +532,7 @@ class Scene {
 
     A3Controller controller = null;
     void setA3Controller0(A3Controller c) {
-        if (canvas!=null)
+        if ((canvas!=null)&&(c!=null))
             c.setA3CanvasInterface(canvas);
         if (universe.scene==this) {
             if (controller!=null) {
@@ -432,9 +541,11 @@ class Scene {
                 controller.stop();
             }
             controller = c;
-            universe.addA3Listener(controller);
-            canvas.addKeyListener(controller);
-            controller.init();
+            if (controller!=null) {
+                universe.addA3Listener(controller);
+                canvas.addKeyListener(controller);
+                controller.init();
+            }
         } else {
             controller = c;
         }
@@ -442,6 +553,8 @@ class Scene {
     /**
      * A3Controllerをセットします。これをセットするとナビゲーションモードが
      * USERに自動的にセットされるので、以前設定していたモードは無効になります。
+     * また，設定されているA3Controllerを解除して無効にするためには引数に
+     * nullを指定して呼び出して下さい．
      */
     public void setA3Controller(A3Controller c) {
         naviMode = NaviMode.USER;
